(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core'), require('@angular/forms')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/common', '@angular/core', '@angular/forms'], factory) :
	(factory((global['ngx-datatable-bootstrap4'] = {}),global.ng.common,global.ng.core,global.ng.forms));
}(this, (function (exports,common,core,forms) { 'use strict';

var DataTableColumn = /** @class */ (function () {
    function DataTableColumn() {
        // for [ngClass]
        this.styleClassObject = {};
        this.sortable = false;
        this.resizable = false;
        this.visible = true;
    }
    /**
     * @param {?} row
     * @param {?} index
     * @return {?}
     */
    DataTableColumn.prototype.getCellColor = function (row, index) {
        if (this.cellColors !== undefined) {
            return ((this.cellColors))(row.item, row, this, index);
        }
    };
    /**
     * @return {?}
     */
    DataTableColumn.prototype.ngOnInit = function () {
        this._initCellClass();
    };
    /**
     * @return {?}
     */
    DataTableColumn.prototype._initCellClass = function () {
        if (!this.styleClass && this.property) {
            if (/^[a-zA-Z0-9_]+$/.test(this.property)) {
                this.styleClass = 'column-' + this.property;
            }
            else {
                this.styleClass = 'column-' + this.property.replace(/[^a-zA-Z0-9_]/g, '');
            }
        }
        if (this.styleClass != null) {
            this.styleClassObject = (_a = {}, _a[this.styleClass] = true, _a);
        }
        var _a;
    };
    return DataTableColumn;
}());
DataTableColumn.decorators = [
    { type: core.Directive, args: [{
                selector: 'data-table-column'
            },] },
];
/**
 * @nocollapse
 */
DataTableColumn.ctorParameters = function () { return []; };
DataTableColumn.propDecorators = {
    'header': [{ type: core.Input },],
    'sortable': [{ type: core.Input },],
    'resizable': [{ type: core.Input },],
    'property': [{ type: core.Input },],
    'styleClass': [{ type: core.Input },],
    'cellColors': [{ type: core.Input },],
    'width': [{ type: core.Input },],
    'visible': [{ type: core.Input },],
    'cellTemplate': [{ type: core.ContentChild, args: ['cellTemplate',] },],
    'headerTemplate': [{ type: core.ContentChild, args: ['headerTemplate',] },],
};
var DataTablePagination = /** @class */ (function () {
    /**
     * @param {?} dataTable
     */
    function DataTablePagination(dataTable) {
        this.dataTable = dataTable;
    }
    /**
     * @return {?}
     */
    DataTablePagination.prototype.pageBack = function () {
        this.dataTable.offset -= Math.min(this.dataTable.limit, this.dataTable.offset);
    };
    /**
     * @return {?}
     */
    DataTablePagination.prototype.pageForward = function () {
        this.dataTable.offset += this.dataTable.limit;
    };
    /**
     * @return {?}
     */
    DataTablePagination.prototype.pageFirst = function () {
        this.dataTable.offset = 0;
    };
    /**
     * @return {?}
     */
    DataTablePagination.prototype.pageLast = function () {
        this.dataTable.offset = (this.maxPage - 1) * this.dataTable.limit;
    };
    Object.defineProperty(DataTablePagination.prototype, "maxPage", {
        /**
         * @return {?}
         */
        get: function () {
            return Math.ceil(this.dataTable.itemCount / this.dataTable.limit);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTablePagination.prototype, "limit", {
        /**
         * @return {?}
         */
        get: function () {
            return this.dataTable.limit;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            // TODO better way to handle that value of number <input> is string?
            this.dataTable.limit = Number(/** @type {?} */ (value));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTablePagination.prototype, "page", {
        /**
         * @return {?}
         */
        get: function () {
            return this.dataTable.page;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this.dataTable.page = Number(/** @type {?} */ (value));
        },
        enumerable: true,
        configurable: true
    });
    return DataTablePagination;
}());
DataTablePagination.decorators = [
    { type: core.Component, args: [{
                selector: 'data-table-pagination',
                template: "\n    <div class=\"navbar p-0 pagination-component\">\n      <div>\n        {{dataTable.translations.paginationRange}}:&nbsp;{{dataTable.offset + 1}}&nbsp;-\n        {{[dataTable.offset + dataTable.limit , dataTable.itemCount] | min}}&nbsp;/&nbsp;{{dataTable.itemCount}}\n      </div>\n      <ul class=\"nav justify-content-end\">\n        <li class=\"nav-item pagination-limit\">\n          <div class=\"input-group\">\n            <div class=\"input-group-prepend\">\n              <span class=\"input-group-text\">{{dataTable.translations.paginationLimit}}:</span>\n            </div>\n            <input #limitInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\"\n                   [ngModel]=\"limit\" (blur)=\"limit = limitInput.value\"\n                   (keyup.enter)=\"limit = limitInput.value\" (keyup.esc)=\"limitInput.value = limit\"/>\n          </div>\n        </li>\n        <li class=\"nav-item\">\n          <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageFirst()\" class=\"btn btn-default\">&laquo;</button>\n          <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageBack()\" class=\"btn btn-default\">&lsaquo;</button>\n          <div class=\"d-inline-block\">\n            <div class=\"input-group\">\n              <input #pageInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\" max=\"{{maxPage}}\"\n                     [ngModel]=\"page\" (blur)=\"page = pageInput.value\"\n                     (keyup.enter)=\"page = pageInput.value\" (keyup.esc)=\"pageInput.value = page\"/>\n              <div class=\"input-group-append\">\n                <span class=\"input-group-text\">/&nbsp;{{dataTable.lastPage}}</span>\n              </div>\n            </div>\n          </div>\n          <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageForward()\"\n                  class=\"btn btn-default\">&rsaquo;\n          </button>\n          <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageLast()\"\n                  class=\"btn btn-default\">&raquo;\n          </button>\n        </li>\n      </ul>\n    </div>\n  ",
                styles: ["\n    .pagination-component .pagination-limit {\n      margin-right: 1rem; }\n      .pagination-component .pagination-limit input {\n        width: 75px; }\n\n    .pagination-component .btn {\n      margin-top: -3px; }\n  "]
            },] },
];
/**
 * @nocollapse
 */
DataTablePagination.ctorParameters = function () { return [
    { type: DataTable$$1, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTable$$1; }),] },] },
]; };
var DataTableRow$$1 = /** @class */ (function () {
    /**
     * @param {?} dataTable
     */
    function DataTableRow$$1(dataTable) {
        this.dataTable = dataTable;
        this.selectedChange = new core.EventEmitter();
        this._this = this;
    }
    Object.defineProperty(DataTableRow$$1.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this._selected;
        },
        /**
         * @param {?} selected
         * @return {?}
         */
        set: function (selected) {
            this._selected = selected;
            this.selectedChange.emit(selected);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTableRow$$1.prototype, "displayIndex", {
        /**
         * @return {?}
         */
        get: function () {
            if (this.dataTable.pagination) {
                return this.dataTable.displayParams.offset + this.index + 1;
            }
            else {
                return this.index + 1;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    DataTableRow$$1.prototype.getTooltip = function () {
        if (this.dataTable.rowTooltip) {
            return this.dataTable.rowTooltip(this.item, this, this.index);
        }
        return '';
    };
    /**
     * @return {?}
     */
    DataTableRow$$1.prototype.ngOnDestroy = function () {
        this.selected = false;
    };
    return DataTableRow$$1;
}());
DataTableRow$$1.decorators = [
    { type: core.Component, args: [{
                selector: '[dataTableRow]',
                template: "\n    <tr class=\"data-table-row\"\n        [title]=\"getTooltip()\"\n        [style.background-color]=\"dataTable.getRowColor(item, index, _this)\"\n        [class.row-odd]=\"index % 2 === 0\"\n        [class.row-even]=\"index % 2 === 1\"\n        [class.selected]=\"selected\"\n        [class.clickable]=\"dataTable.selectOnRowClick\"\n        (dblclick)=\"dataTable.rowDoubleClicked(_this, $event)\"\n        (click)=\"dataTable.rowClicked(_this, $event)\">\n      <td [hide]=\"!dataTable.expandColumnVisible\" (click)=\"expanded = !expanded; $event.stopPropagation()\"\n          class=\"row-expand-button\">\n        <span class=\"fa fa-arrow-circle-o-right\" [hide]=\"expanded\"></span>\n        <span class=\"fa fa-arrow-circle-o-down\" [hide]=\"!expanded\"></span>\n      </td>\n      <td [hide]=\"!dataTable.indexColumnVisible\" class=\"index-column\" [textContent]=\"displayIndex\"></td>\n      <td [hide]=\"!dataTable.selectColumnVisible\" class=\"select-column\">\n        <input type=\"checkbox\" [(ngModel)]=\"selected\"/>\n      </td>\n      <td *ngFor=\"let column of dataTable.columns\" [hide]=\"!column.visible\" [ngClass]=\"column.styleClassObject\"\n          class=\"data-column\"\n          [style.background-color]=\"column.getCellColor(_this, index)\"\n          (click)=\"dataTable.cellClicked(column, _this, $event)\">\n        <div *ngIf=\"!column.cellTemplate\" [textContent]=\"item[column.property]\"></div>\n        <ng-container *ngIf=\"column.cellTemplate\" [ngTemplateOutlet]=\"column.cellTemplate\"\n             [ngTemplateOutletContext]=\"{column: column, row: _this, item: item}\"></ng-container>\n      </td>\n    </tr>\n    <tr *ngIf=\"dataTable.expandableRows\" [hide]=\"!expanded\" class=\"row-expansion\">\n      <td [attr.colspan]=\"dataTable.columnCount\">\n        <ng-container [ngTemplateOutlet]=\"dataTable.expandTemplate\"\n                      [ngTemplateOutletContext]=\"{row: _this, item: item}\"></ng-container>\n      </td>\n    </tr>\n  ",
                styles: ["\n    .select-column {\n      text-align: center; }\n\n    .row-expand-button {\n      cursor: pointer;\n      text-align: center; }\n\n    .clickable {\n      cursor: pointer; }\n  "]
            },] },
];
/**
 * @nocollapse
 */
DataTableRow$$1.ctorParameters = function () { return [
    { type: DataTable$$1, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTable$$1; }),] },] },
]; };
DataTableRow$$1.propDecorators = {
    'item': [{ type: core.Input },],
    'index': [{ type: core.Input },],
    'selectedChange': [{ type: core.Output },],
};
/**
 * @param {?} event
 * @param {?} __1
 * @return {?}
 */
function drag(event, _a) {
    var move = _a.move, up = _a.up;
    var /** @type {?} */ startX = event.pageX;
    var /** @type {?} */ startY = event.pageY;
    var /** @type {?} */ x = startX;
    var /** @type {?} */ y = startY;
    var /** @type {?} */ moved = false;
    /**
     * @param {?} e
     * @return {?}
     */
    function mouseMoveHandler(e) {
        var /** @type {?} */ dx = e.pageX - x;
        var /** @type {?} */ dy = e.pageY - y;
        x = e.pageX;
        y = e.pageY;
        if (dx || dy) {
            moved = true;
        }
        move(e, dx, dy, x, y);
        e.preventDefault(); // to avoid text selection
    }
    /**
     * @param {?} e
     * @return {?}
     */
    function mouseUpHandler(e) {
        x = e.pageX;
        y = e.pageY;
        document.removeEventListener('mousemove', mouseMoveHandler);
        document.removeEventListener('mouseup', mouseUpHandler);
        if (up) {
            up(e, x, y, moved);
        }
    }
    document.addEventListener('mousemove', mouseMoveHandler);
    document.addEventListener('mouseup', mouseUpHandler);
}
var defaultTranslations = ({
    indexColumn: 'index',
    selectColumn: 'select',
    expandColumn: 'expand',
    paginationLimit: 'Limit',
    paginationRange: 'Results'
});
var DataTable$$1 = /** @class */ (function () {
    function DataTable$$1() {
        this._sortAsc = true;
        this._offset = 0;
        this._limit = 10;
        this._items = [];
        this._scheduledReload = null;
        this._selectAllCheckbox = false;
        this._displayParams = ({});
        this._reloading = false;
        this._resizeInProgress = false;
        this.selectedRows = [];
        this.resizeLimit = 30;
        this.pagination = true;
        this.indexColumn = true;
        this.indexColumnHeader = '';
        this.selectColumn = false;
        this.multiSelect = true;
        this.substituteRows = true;
        this.expandableRows = false;
        this.translations = defaultTranslations;
        this.selectOnRowClick = false;
        this.autoReload = true;
        this.showReloading = false;
        // event handlers:
        this.rowClick = new core.EventEmitter();
        this.rowDoubleClick = new core.EventEmitter();
        this.headerClick = new core.EventEmitter();
        this.cellClick = new core.EventEmitter();
        this.reload = new core.EventEmitter();
    }
    Object.defineProperty(DataTable$$1.prototype, "items", {
        /**
         * @return {?}
         */
        get: function () {
            return this._items;
        },
        /**
         * @param {?} items
         * @return {?}
         */
        set: function (items) {
            this._items = items;
            this._onReloadFinished();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "sortBy", {
        /**
         * @return {?}
         */
        get: function () {
            return this._sortBy;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._sortBy = value;
            this._triggerReload();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "sortAsc", {
        /**
         * @return {?}
         */
        get: function () {
            return this._sortAsc;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._sortAsc = value;
            this._triggerReload();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "offset", {
        /**
         * @return {?}
         */
        get: function () {
            return this._offset;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._offset = value;
            this._triggerReload();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "limit", {
        /**
         * @return {?}
         */
        get: function () {
            return this._limit;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._limit = value;
            this._triggerReload();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "page", {
        /**
         * @return {?}
         */
        get: function () {
            return Math.floor(this.offset / this.limit) + 1;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this.offset = (value - 1) * this.limit;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "lastPage", {
        /**
         * @return {?}
         */
        get: function () {
            return Math.ceil(this.itemCount / this.limit);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "reloading", {
        /**
         * @return {?}
         */
        get: function () {
            return this._reloading;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "displayParams", {
        /**
         * @return {?}
         */
        get: function () {
            return this._displayParams;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "selectAllCheckbox", {
        /**
         * @return {?}
         */
        get: function () {
            return this._selectAllCheckbox;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._selectAllCheckbox = value;
            this._onSelectAllChanged(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "columnCount", {
        /**
         * @return {?}
         */
        get: function () {
            var /** @type {?} */ count = 0;
            count += this.indexColumnVisible ? 1 : 0;
            count += this.selectColumnVisible ? 1 : 0;
            count += this.expandColumnVisible ? 1 : 0;
            this.columns.toArray().forEach(function (column) {
                count += column.visible ? 1 : 0;
            });
            return count;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataTable$$1.prototype, "substituteItems", {
        /**
         * @return {?}
         */
        get: function () {
            return Array.from({ length: /** @type {?} */ ((this.displayParams)).limit - this.items.length });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} item
     * @param {?} index
     * @param {?} row
     * @return {?}
     */
    DataTable$$1.prototype.getRowColor = function (item, index, row) {
        if (this.rowColors !== undefined) {
            return ((this.rowColors))(item, row, index);
        }
    };
    /**
     * @param {?} sortBy
     * @param {?} asc
     * @return {?}
     */
    DataTable$$1.prototype.sort = function (sortBy, asc) {
        this.sortBy = sortBy;
        this.sortAsc = asc;
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype.reloadItems = function () {
        this._reloading = true;
        this.reload.emit(this._getRemoteParameters());
    };
    /**
     * @param {?} row
     * @param {?} event
     * @return {?}
     */
    DataTable$$1.prototype.rowClicked = function (row, event) {
        this.rowClick.emit({ row: row, event: event });
    };
    /**
     * @param {?} row
     * @param {?} event
     * @return {?}
     */
    DataTable$$1.prototype.rowDoubleClicked = function (row, event) {
        this.rowDoubleClick.emit({ row: row, event: event });
    };
    /**
     * @param {?} column
     * @param {?} event
     * @return {?}
     */
    DataTable$$1.prototype.headerClicked = function (column, event) {
        if (!this._resizeInProgress) {
            this.headerClick.emit({ column: column, event: event });
        }
        else {
            // this is because I can't prevent click from mousup of the drag end
            this._resizeInProgress = false;
        }
    };
    /**
     * @param {?} column
     * @param {?} row
     * @param {?} event
     * @return {?}
     */
    DataTable$$1.prototype.cellClicked = function (column, row, event) {
        this.cellClick.emit({ row: row, column: column, event: event });
    };
    /**
     * @param {?} row
     * @return {?}
     */
    DataTable$$1.prototype.onRowSelectChanged = function (row) {
        // maintain the selectedRow(s) view
        if (this.multiSelect) {
            var /** @type {?} */ index = this.selectedRows.indexOf(row);
            if (row.selected && index < 0) {
                this.selectedRows.push(row);
            }
            else if (!row.selected && index >= 0) {
                this.selectedRows.splice(index, 1);
            }
        }
        else {
            if (row.selected) {
                this.selectedRow = row;
            }
            else if (this.selectedRow === row) {
                this.selectedRow = undefined;
            }
        }
        // unselect all other rows:
        if (row.selected && !this.multiSelect) {
            this.rows.toArray().filter(function (row_) { return row_.selected; }).forEach(function (row_) {
                if (row_ !== row) {
                    row_.selected = false;
                }
            });
        }
    };
    /**
     * @param {?} event
     * @param {?} column
     * @param {?} columnElement
     * @return {?}
     */
    DataTable$$1.prototype.resizeColumnStart = function (event, column, columnElement) {
        var _this = this;
        this._resizeInProgress = true;
        drag(event, {
            move: function (moveEvent, dx) {
                if (_this._isResizeInLimit(columnElement, dx)) {
                    column.width = columnElement.offsetWidth + dx;
                }
            },
        });
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype.ngOnInit = function () {
        this._initDefaultValues();
        this._initDefaultClickEvents();
        this._updateDisplayParams();
        if (this.autoReload && this._scheduledReload == null) {
            this.reloadItems();
        }
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._initDefaultValues = function () {
        this.indexColumnVisible = this.indexColumn;
        this.selectColumnVisible = this.selectColumn;
        this.expandColumnVisible = this.expandableRows;
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._initDefaultClickEvents = function () {
        var _this = this;
        this.headerClick.subscribe(function (tableEvent) { return _this._sortColumn(tableEvent.column); });
        if (this.selectOnRowClick) {
            this.rowClick.subscribe(function (tableEvent) { return tableEvent.row.selected = !tableEvent.row.selected; });
        }
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._onReloadFinished = function () {
        this._updateDisplayParams();
        this._selectAllCheckbox = false;
        this._reloading = false;
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._updateDisplayParams = function () {
        this._displayParams = {
            sortBy: this.sortBy,
            sortAsc: this.sortAsc,
            offset: this.offset,
            limit: this.limit
        };
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._triggerReload = function () {
        var _this = this;
        // for avoiding cascading reloads if multiple params are set at once:
        if (this._scheduledReload) {
            clearTimeout(this._scheduledReload);
        }
        this._scheduledReload = setTimeout(function () {
            _this.reloadItems();
        });
    };
    /**
     * @return {?}
     */
    DataTable$$1.prototype._getRemoteParameters = function () {
        var /** @type {?} */ params = ({});
        if (this.sortBy) {
            params.sortBy = this.sortBy;
            params.sortAsc = this.sortAsc;
        }
        if (this.pagination) {
            params.offset = this.offset;
            params.limit = this.limit;
        }
        return params;
    };
    /**
     * @param {?} column
     * @return {?}
     */
    DataTable$$1.prototype._sortColumn = function (column) {
        if (column.sortable) {
            var /** @type {?} */ ascending = this.sortBy === column.property ? !this.sortAsc : true;
            this.sort(column.property, ascending);
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    DataTable$$1.prototype._onSelectAllChanged = function (value) {
        this.rows.toArray().forEach(function (row) { return row.selected = value; });
    };
    /**
     * @param {?} columnElement
     * @param {?} dx
     * @return {?}
     */
    DataTable$$1.prototype._isResizeInLimit = function (columnElement, dx) {
        /* This is needed because CSS min-width didn't work on table-layout: fixed.
         Without the limits, resizing can make the next column disappear completely,
         and even increase the table width. The current implementation suffers from the fact,
         that offsetWidth sometimes contains out-of-date values. */
        return !((dx < 0 && (columnElement.offsetWidth + dx) <= this.resizeLimit) ||
            !columnElement.nextElementSibling || // resizing doesn't make sense for the last visible column
            (dx >= 0 && (((columnElement.nextElementSibling)).offsetWidth + dx) <= this.resizeLimit));
    };
    return DataTable$$1;
}());
DataTable$$1.decorators = [
    { type: core.Component, args: [{
                selector: 'data-table',
                template: "\n    <div class=\"data-table-wrapper\">\n      <data-table-title *ngIf=\"title\"></data-table-title>\n\n      <div class=\"data-table-box\">\n        <table class=\"table table-condensed data-table\">\n          <thead>\n          <tr>\n            <th [hide]=\"!expandColumnVisible\" class=\"expand-column-header\">\n            <th [hide]=\"!indexColumnVisible\" class=\"index-column-header\">\n              <span [textContent]=\"indexColumnHeader\"></span>\n            </th>\n            <th [hide]=\"!selectColumnVisible\" class=\"select-column-header\">\n              <input [hide]=\"!multiSelect\" type=\"checkbox\" [(ngModel)]=\"selectAllCheckbox\"/>\n            </th>\n            <th *ngFor=\"let column of columns\" #th [hide]=\"!column.visible\" (click)=\"headerClicked(column, $event)\"\n                [class.sortable]=\"column.sortable\" [class.resizable]=\"column.resizable\"\n                [ngClass]=\"column.styleClassObject\" class=\"column-header\" [style.width]=\"column.width | px\">\n              <span *ngIf=\"!column.headerTemplate\" [textContent]=\"column.header\"></span>\n              <ng-container *ngIf=\"column.headerTemplate\" [ngTemplateOutlet]=\"column.headerTemplate\"\n                    [ngTemplateOutletContext]=\"{column: column}\"></ng-container>\n              <span class=\"column-sort-icon\" *ngIf=\"column.sortable\">\n                <i class=\"fa fa-sort column-sortable-icon\" [hide]=\"column.property === sortBy\"></i>\n                <span [hide]=\"column.property !== sortBy\">\n                  <i class=\"fa fa-sort-asc\" [hide]=\"sortAsc\"></i>\n                  <i class=\"fa fa-sort-desc\" [hide]=\"!sortAsc\"></i>\n                </span>\n              </span>\n              <span *ngIf=\"column.resizable\" class=\"column-resize-handle\"\n                    (mousedown)=\"resizeColumnStart($event, column, th)\"></span>\n            </th>\n          </tr>\n          </thead>\n          <tbody *ngFor=\"let item of items; let index=index\" class=\"data-table-row-wrapper\"\n                 dataTableRow #row [item]=\"item\" [index]=\"index\" (selectedChange)=\"onRowSelectChanged(row)\">\n          </tbody>\n          <tbody class=\"substitute-rows\" *ngIf=\"pagination && substituteRows\">\n          <tr *ngFor=\"let item of substituteItems, let index = index\"\n              [class.row-odd]=\"(index + items.length) % 2 === 0\"\n              [class.row-even]=\"(index + items.length) % 2 === 1\">\n            <td [hide]=\"!expandColumnVisible\"></td>\n            <td [hide]=\"!indexColumnVisible\">&nbsp;</td>\n            <td [hide]=\"!selectColumnVisible\"></td>\n            <td *ngFor=\"let column of columns\" [hide]=\"!column.visible\">\n          </tr>\n          </tbody>\n        </table>\n        <div class=\"loading-cover\" *ngIf=\"showReloading && reloading\"></div>\n      </div>\n\n      <data-table-pagination *ngIf=\"pagination\"></data-table-pagination>\n    </div>\n  ",
                styles: ["\n    /* bootstrap override: */\n    :host /deep/ .data-table.table > tbody + tbody {\n      border-top: none; }\n\n    :host /deep/ .data-table.table td {\n      vertical-align: middle; }\n\n    :host /deep/ .data-table > thead > tr > th,\n    :host /deep/ .data-table > tbody > tr > td {\n      overflow: hidden; }\n\n    /* I can't use the bootstrap striped table, because of the expandable rows */\n    :host /deep/ .row-odd {\n      background-color: #F6F6F6; }\n\n    .data-table .substitute-rows > tr:hover,\n    :host /deep/ .data-table .data-table-row:hover {\n      background-color: #ECECEC; }\n\n    /* table itself: */\n    .data-table {\n      -webkit-box-shadow: 0 0 15px #ececec;\n              box-shadow: 0 0 15px #ececec;\n      table-layout: fixed; }\n\n    /* header cells: */\n    .column-header {\n      position: relative; }\n\n    .expand-column-header {\n      width: 50px; }\n\n    .select-column-header {\n      width: 50px;\n      text-align: center; }\n\n    .index-column-header {\n      width: 40px; }\n\n    .column-header.sortable {\n      cursor: pointer; }\n\n    .column-header .column-sort-icon {\n      float: right; }\n\n    .column-header.resizable .column-sort-icon {\n      margin-right: 8px; }\n\n    .column-header .column-sort-icon .column-sortable-icon {\n      color: lightgray; }\n\n    .column-header .column-resize-handle {\n      position: absolute;\n      top: 0;\n      right: 0;\n      margin: 0;\n      padding: 0;\n      width: 8px;\n      height: 100%;\n      cursor: col-resize; }\n\n    /* cover: */\n    .data-table-box {\n      position: relative; }\n\n    .loading-cover {\n      position: absolute;\n      width: 100%;\n      height: 100%;\n      background-color: rgba(255, 255, 255, 0.3);\n      top: 0; }\n  "]
            },] },
];
/**
 * @nocollapse
 */
DataTable$$1.ctorParameters = function () { return []; };
DataTable$$1.propDecorators = {
    'itemCount': [{ type: core.Input },],
    'pagination': [{ type: core.Input },],
    'indexColumn': [{ type: core.Input },],
    'indexColumnHeader': [{ type: core.Input },],
    'rowColors': [{ type: core.Input },],
    'rowTooltip': [{ type: core.Input },],
    'selectColumn': [{ type: core.Input },],
    'multiSelect': [{ type: core.Input },],
    'substituteRows': [{ type: core.Input },],
    'expandableRows': [{ type: core.Input },],
    'translations': [{ type: core.Input },],
    'selectOnRowClick': [{ type: core.Input },],
    'autoReload': [{ type: core.Input },],
    'showReloading': [{ type: core.Input },],
    'rowClick': [{ type: core.Output },],
    'rowDoubleClick': [{ type: core.Output },],
    'headerClick': [{ type: core.Output },],
    'cellClick': [{ type: core.Output },],
    'reload': [{ type: core.Output },],
    'title': [{ type: core.ContentChild, args: [core.forwardRef(function () { return DataTableTitle; }),] },],
    'columns': [{ type: core.ContentChildren, args: [DataTableColumn,] },],
    'rows': [{ type: core.ViewChildren, args: [DataTableRow$$1,] },],
    'expandTemplate': [{ type: core.ContentChild, args: ['expandTemplate',] },],
    'items': [{ type: core.Input },],
    'sortBy': [{ type: core.Input },],
    'sortAsc': [{ type: core.Input },],
    'offset': [{ type: core.Input },],
    'limit': [{ type: core.Input },],
    'page': [{ type: core.Input },],
};
var DataTableTitleComponent = /** @class */ (function () {
    /**
     * @param {?} dataTable
     */
    function DataTableTitleComponent(dataTable) {
        this.dataTable = dataTable;
        this.columnSelectorOpen = false;
    }
    /**
     * @return {?}
     */
    DataTableTitleComponent.prototype._closeSelector = function () {
        this.columnSelectorOpen = false;
    };
    return DataTableTitleComponent;
}());
DataTableTitleComponent.decorators = [
    { type: core.Component, args: [{
                selector: 'data-table-title',
                template: "\n    <div class=\"navbar\">\n      <h4 class=\"title\" [textContent]=\"dataTable.title.title\"></h4>\n      <ng-container *ngIf=\"dataTable.title.titleTemplate\" [ngTemplateOutlet]=\"dataTable.title.titleTemplate\"\n                    [ngTemplateOutletContext]=\"{items: dataTable.items}\">\n      </ng-container>\n      <div *ngIf=\"dataTable.title.controls\">\n        <button type=\"button\" class=\"btn btn-default btn-sm refresh-button\" (click)=\"dataTable.reloadItems()\">\n          <i class=\"fa fa-refresh\"></i>\n        </button>\n        <button type=\"button\" class=\"btn btn-default btn-sm column-selector-button\" [class.active]=\"columnSelectorOpen\"\n                (click)=\"columnSelectorOpen = !columnSelectorOpen; $event.stopPropagation()\">\n          <i class=\"fa fa-list\"></i>\n        </button>\n        <div class=\"column-selector-wrapper\" (click)=\"$event.stopPropagation()\">\n          <div *ngIf=\"columnSelectorOpen\" class=\"column-selector-box card\">\n            <div *ngIf=\"dataTable.expandableRows\" class=\"column-selector-fixed-column checkbox\">\n              <label>\n                <input type=\"checkbox\" [(ngModel)]=\"dataTable.expandColumnVisible\"/>\n                <span>{{dataTable.translations.expandColumn}}</span>\n              </label>\n            </div>\n            <div *ngIf=\"dataTable.indexColumn\" class=\"column-selector-fixed-column checkbox\">\n              <label>\n                <input type=\"checkbox\" [(ngModel)]=\"dataTable.indexColumnVisible\"/>\n                <span>{{dataTable.translations.indexColumn}}</span>\n              </label>\n            </div>\n            <div *ngIf=\"dataTable.selectColumn\" class=\"column-selector-fixed-column checkbox\">\n              <label>\n                <input type=\"checkbox\" [(ngModel)]=\"dataTable.selectColumnVisible\"/>\n                <span>{{dataTable.translations.selectColumn}}</span>\n              </label>\n            </div>\n            <div *ngFor=\"let column of dataTable.columns\" class=\"column-selector-column checkbox\">\n              <label>\n                <input type=\"checkbox\" [(ngModel)]=\"column.visible\"/>\n                <span [textContent]=\"column.header\"></span>\n              </label>\n            </div>\n          </div>\n        </div>\n      </div>\n    </div>\n  ",
                styles: ["\n    .navbar {\n      padding: 10px 0; }\n      .navbar .title {\n        margin: 5px 0 0 5px; }\n\n    .column-selector-wrapper {\n      position: relative; }\n      .column-selector-wrapper .column-selector-box {\n        -webkit-box-shadow: 0 0 10px lightgray;\n                box-shadow: 0 0 10px lightgray;\n        width: 150px;\n        padding: 10px;\n        position: absolute;\n        right: 0;\n        top: 1px;\n        z-index: 1060; }\n        .column-selector-wrapper .column-selector-box .checkbox {\n          margin-bottom: 4px; }\n        .column-selector-wrapper .column-selector-box .column-selector-fixed-column {\n          font-style: italic; }\n  "]
            },] },
];
/**
 * @nocollapse
 */
DataTableTitleComponent.ctorParameters = function () { return [
    { type: DataTable$$1, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTable$$1; }),] },] },
]; };
DataTableTitleComponent.propDecorators = {
    '_closeSelector': [{ type: core.HostListener, args: ['document:click', ['$event'],] },],
};
var DataTableTitle = /** @class */ (function () {
    function DataTableTitle() {
        this.title = '';
        this.controls = true;
    }
    return DataTableTitle;
}());
DataTableTitle.decorators = [
    { type: core.Directive, args: [{
                selector: 'data-table-title'
            },] },
];
/**
 * @nocollapse
 */
DataTableTitle.ctorParameters = function () { return []; };
DataTableTitle.propDecorators = {
    'title': [{ type: core.Input },],
    'titleTemplate': [{ type: core.ContentChild, args: ['titleTemplate',] },],
    'controls': [{ type: core.Input },],
};
/**
 * @param {?} obj
 * @return {?}
 */
function isBlank(obj) {
    return obj === undefined || obj === null;
}
var HideDirective = /** @class */ (function () {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    function HideDirective(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._prevCondition = null;
    }
    Object.defineProperty(HideDirective.prototype, "hide", {
        /**
         * @param {?} newCondition
         * @return {?}
         */
        set: function (newCondition) {
            this.initDisplayStyle();
            if (newCondition && (isBlank(this._prevCondition) || !this._prevCondition)) {
                this._prevCondition = true;
                this._renderer.setElementStyle(this._elementRef.nativeElement, 'display', 'none');
            }
            else if (!newCondition && (isBlank(this._prevCondition) || this._prevCondition)) {
                this._prevCondition = false;
                this._renderer.setElementStyle(this._elementRef.nativeElement, 'display', this._displayStyle);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    HideDirective.prototype.initDisplayStyle = function () {
        if (this._displayStyle === undefined) {
            var /** @type {?} */ displayStyle = this._elementRef.nativeElement.style.display;
            if (displayStyle && displayStyle !== 'none') {
                this._displayStyle = displayStyle;
            }
        }
    };
    return HideDirective;
}());
HideDirective.decorators = [
    { type: core.Directive, args: [{ selector: '[hide]' },] },
];
/**
 * @nocollapse
 */
HideDirective.ctorParameters = function () { return [
    { type: core.ElementRef, },
    { type: core.Renderer, },
]; };
HideDirective.propDecorators = {
    'hide': [{ type: core.Input },],
};
var MinPipe = /** @class */ (function () {
    function MinPipe() {
    }
    /**
     * @param {?} value
     * @param {?} args
     * @return {?}
     */
    MinPipe.prototype.transform = function (value, args) {
        return Math.min.apply(null, value);
    };
    return MinPipe;
}());
MinPipe.decorators = [
    { type: core.Pipe, args: [{
                name: 'min'
            },] },
];
/**
 * @nocollapse
 */
MinPipe.ctorParameters = function () { return []; };
var PxPipe = /** @class */ (function () {
    function PxPipe() {
    }
    /**
     * @param {?} value
     * @param {?} args
     * @return {?}
     */
    PxPipe.prototype.transform = function (value, args) {
        if (value === undefined) {
            return;
        }
        if (typeof value === 'string') {
            return value;
        }
        if (typeof value === 'number') {
            return value + 'px';
        }
    };
    return PxPipe;
}());
PxPipe.decorators = [
    { type: core.Pipe, args: [{
                name: 'px'
            },] },
];
/**
 * @nocollapse
 */
PxPipe.ctorParameters = function () { return []; };
var DataTableResource = /** @class */ (function () {
    /**
     * @param {?} items
     */
    function DataTableResource(items) {
        this.items = items;
    }
    /**
     * @param {?} params
     * @param {?=} filter
     * @return {?}
     */
    DataTableResource.prototype.query = function (params, filter) {
        var /** @type {?} */ result = [];
        if (filter) {
            result = this.items.filter(filter);
        }
        else {
            result = this.items.slice(); // shallow copy to use for sorting instead of changing the original
        }
        if (params.sortBy) {
            result.sort(function (a, b) {
                if (typeof a[params.sortBy] === 'string') {
                    return a[params.sortBy].localeCompare(b[params.sortBy]);
                }
                else {
                    return a[params.sortBy] - b[params.sortBy];
                }
            });
            if (params.sortAsc === false) {
                result.reverse();
            }
        }
        if (params.offset !== undefined) {
            if (params.limit === undefined) {
                result = result.slice(params.offset, result.length);
            }
            else {
                result = result.slice(params.offset, params.offset + params.limit);
            }
        }
        return new Promise(function (resolve, reject) {
            setTimeout(function () { return resolve(result); });
        });
    };
    /**
     * @return {?}
     */
    DataTableResource.prototype.count = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            setTimeout(function () { return resolve(_this.items.length); });
        });
    };
    return DataTableResource;
}());
var DataTableModule = /** @class */ (function () {
    function DataTableModule() {
    }
    return DataTableModule;
}());
DataTableModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [
                    common.CommonModule,
                    forms.FormsModule
                ],
                declarations: [
                    DataTable$$1,
                    DataTableColumn,
                    DataTableTitle,
                    DataTableRow$$1,
                    DataTablePagination,
                    DataTableTitleComponent,
                    PxPipe,
                    HideDirective,
                    MinPipe
                ],
                exports: [DataTable$$1, DataTableColumn, DataTableTitle]
            },] },
];
/**
 * @nocollapse
 */
DataTableModule.ctorParameters = function () { return []; };

exports.DataTable = DataTable$$1;
exports.DataTableColumn = DataTableColumn;
exports.DataTableRow = DataTableRow$$1;
exports.DataTableTitle = DataTableTitle;
exports.DataTableModule = DataTableModule;
exports.defaultTranslations = defaultTranslations;
exports.DataTableResource = DataTableResource;
exports.ɵd = DataTable$$1;
exports.ɵb = DataTableColumn;
exports.ɵc = DataTableRow$$1;
exports.ɵa = DataTableTitle;
exports.ɵe = DataTablePagination;
exports.ɵf = DataTableTitleComponent;
exports.ɵh = HideDirective;
exports.ɵi = MinPipe;
exports.ɵg = PxPipe;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-datatable-bootstrap4.umd.js.map
